@@ -1,4 +1,4 @@
-sum.set<-function(g,n,p,y,set,active) {
+sumSet<-function(g,n,p,y,set,active) {
 z<-array(0.0,c(n,p))
 for (j in set)
 	if (active[j]){	    
@@ -7,11 +7,11 @@ for (j in set)
 return(z)
 }
 
-pca.update.y<-function(data,x,y,totsum,active,rank,level,nset){
+pcaUpdateY<-function(data,x,y,totsum,active,rank,level,nset){
 for (j in 1:nset){
 	if (active[j]){	    
-		gg<-data[,j]; ycen<-compute.y(gg,x); d<-as.vector(table(gg)); ii<-which(!is.na(gg))
-		yy<-restrict.y(d,ycen,rank[j],level[j])$y
+		gg<-data[,j]; ycen<-computeY(gg,x); d<-as.vector(table(gg)); ii<-which(!is.na(gg))
+		yy<-restrictY(d,ycen,rank[j],level[j])$y
 		y[[j]]<-yy
 		totsum[ii,]<-totsum[ii,]+yy[gg[ii],]
 		}
@@ -19,17 +19,17 @@ for (j in 1:nset){
 return(list(y=y,totsum=totsum))
 }
 
-cca.update.y<-function(data,x,y,totsum,active,rank,level,sets){
+ccaUpdateY<-function(data,x,y,totsum,active,rank,level,sets){
 nobj<-dim(x)[1]; ndim<-dim(x)[2]; nset<-length(sets)
 for (l in 1:nset) {
 	indi<-sets[[l]]
-	ss<-sum.set(data,nobj,ndim,y,indi,active)
+	ss<-sumSet(data,nobj,ndim,y,indi,active)
 	for (j in indi) {
 		if (active[j]){	
 			gg<-data[,j]; yy<-y[[j]]; ii<-which(!is.na(gg)); d<-as.vector(table(gg))
 			ss<-ss-yy[gg[ii],]
-			ycen<-compute.y(data[,j],x-ss)
-			yy<-restrict.y(d,ycen,rank[j],level[j])$y
+			ycen<-computeY(data[,j],x-ss)
+			yy<-restrictY(d,ycen,rank[j],level[j])$y
 			ss<-ss+yy[gg[ii],]
 			y[[j]]<-yy
 			}
@@ -39,28 +39,28 @@ for (l in 1:nset) {
 return(list(y=y,totsum=totsum))
 }
 
-compute.y<-function(g,x) apply(x, 2, function(z) tapply(z,g,mean))
+computeY<-function(g,x) apply(x, 2, function(z) tapply(z,g,mean))
 
-restrict.y<-function(d,y,r,level) {
+restrictY<-function(d,y,r,level) {
 if (sum(y^2) == 0) return(y)
 switch(level,
-"NO"=return(nominal.y(d,y,r)),
-"OR"=return(ordinal.y(d,y,r)),
-"NU"=return(numerical.y(d,y,r)),
-"PO"=return(polynomial.y(d,y,r)))
+"NO"=return(nominalY(d,y,r)),
+"OR"=return(ordinalY(d,y,r)),
+"NU"=return(numericalY(d,y,r)),
+"PO"=return(polynomialY(d,y,r)))
 }
 
-nominal.y<-function(d,y,r) {
+nominalY<-function(d,y,r) {
 qq<-La.svd(sqrt(d)*y,r,r)
 zz<-(1/sqrt(d))*qq$u
 aa<-qq$d[1:r]*qq$vt
 list(y=zz%*%aa,z=zz,a=aa)
 }
 
-ordinal.y<-function(d,y,r,itermax=100,eps=1e-6) {
+ordinalY<-function(d,y,r,itermax=100,eps=1e-6) {
 qq<-La.svd(sqrt(d)*y,r,r)
 a1=cbind(qq$vt[1,])
-z1=cbind(orthogonal.polynomials(d,1:length(d),1))
+z1=cbind(orthogonalPolynomials(d,1:length(d),1))
 if (r > 1) {
 	a2<-cbind(qq$vt[2:r,])
 	z2<-cbind(qq$u[,2:r])}
@@ -88,13 +88,13 @@ repeat{
 		else {iter<-iter+1; sold<-snew}
 }
 if (r > 1) z<-cbind(z1,z2) else z<-cbind(z1)
-z<-weighted.gram.schmidt(z,d)$pol
+z<-weightedGramSchmidt(z,d)$pol
 a<-crossprod(y,d*z)
 list(yhat=z%*%t(a),z=z,a=a)
 }
 
-numerical.y<-function(d,y,r) {
-z0<-orthogonal.polynomials(d,1:length(d),1)
+numericalY<-function(d,y,r) {
+z0<-orthogonalPolynomials(d,1:length(d),1)
 a0<-as.vector(crossprod(z0*d,y))
 if (r == 1)
 	return(list(y=z0%o%a0,z=cbind(z0),a=rbind(a0)))
@@ -106,9 +106,9 @@ else {
 	return(list(y=zz%*%aa,z=zz,a=aa))}
 }
 
-polynomial.y<-function(d,y,r) {
+polynomialY<-function(d,y,r) {
 k<-length(d)
-zz<-orthogonal.polynomials(d,1:k,min(r,k-1))
+zz<-orthogonalPolynomials(d,1:k,min(r,k-1))
 aa<-crossprod(zz,d*y)
 list(y=zz%*%aa,z=zz,a=aa)
 }
@@ -137,14 +137,41 @@ abline(h=0)
 starplot<-function(name,vname,g,y,x,s,t) {
 plot(x,col="GREEN",pch=8,main=paste("Starplot for",name,":",vname),
 xlab=paste("dimension",s),ylab=paste("dimension",t))
-z<-compute.y(g, x)
+z<-computeY(g, x)
 points(z,type="n")
 text(z,levels(g),col="RED")
 for (i in 1:length(g)) lines(rbind(x[i,],z[g[i],]),col="BLUE")
 }
 
+spanplot<-function(name,vname,g,y,x,s,t) {
+plot(x,col="GREEN",pch=8,main=paste("Spanplot for",name,":",vname),
+xlab=paste("dimension",s),ylab=paste("dimension",t))
+lev<-levels(g)
+rb<-rainbow(length(lev))
+for (k in lev) {
+	ind<-which(k==g)
+	n<-length(ind)
+	mm<-mst(dist(x[ind,]))
+	for (i in 1:n) {
+		jnd<-which(1==as.vector(mm[i,]))
+		sapply(jnd,function(r) lines(rbind(x[ind[i],],x[ind[r],]),col=rb[which(lev==k)]))
+		}
+	}
+}
+
+hullplot<-function(name,vname,g,y,x,s,t) {
+plot(x,col="GREEN",pch=8,main=paste("Hullplot for",name,":",vname),
+xlab=paste("dimension",s),ylab=paste("dimension",t))
+for (k in levels(g)) {
+	ind<-which(k==g)
+	lst<-ind[chull(x[ind,])]
+	lines(x[c(lst,lst[1]),])
+	text(x[lst,],k) 
+	}
+}
+
 lossplot<-function(name,vname,g,y,x,s,t) {
-z<-compute.y(g, x); k<-dim(z)[1]
+z<-computeY(g, x); k<-dim(z)[1]
 xx<-c(min(c(z[,1],y[,1])),max(c(z[,1],y[,1])))
 yy<-c(min(c(z[,2],y[,2])),max(c(z[,2],y[,2])))
 plot(y,type="n",main=paste("Lossplot for",name,":",vname),
@@ -161,7 +188,7 @@ graphplot<-function(name,g,x,s,t) {
 plot(x,col="GREEN",pch=8,main=paste("Graphplot for",name),
 xlab=paste("dimension",s),ylab=paste("dimension",t))
 for (j in 1:dim(g)[2]){
-	y<-compute.y(g[,j], x)
+	y<-computeY(g[,j], x)
 	points(y,col="RED",pch=16)
 	for (i in 1:dim(g)[1]) lines(rbind(x[i,],y[g[i,j],]))}
 }
@@ -178,7 +205,7 @@ plot(x,col="GREEN",pch=8,main=paste("Object score plot for",name),
 xlab=paste("dimension",s),ylab=paste("dimension",t))
 }
 
-write.head<-function(name,vname,p,a,r,c,ofile) {
+writeHead<-function(name,vname,p,a,r,c,ofile) {
 s<-NULL; sl<-35+10*p; for (i in 1:sl) s<-paste(s,"*",sep="")
 cat("\n",formatC(s,format="s"),"\n",file=ofile,sep="")
 cat(formatC(vname,format="s"),
@@ -190,7 +217,7 @@ switch(c,"NO"="nominal","OR"="ordinal","NU"="numerical","PO"="polynomial"),
 cat(formatC(s,format="s"),"\n",file=ofile)
 }
 
-write.y<-function(g,y,t,ofile) {
+writeY<-function(g,y,t,ofile) {
 d<-as.vector(table(g));l<-levels(g)
 s<-NULL; sl<-35+10*dim(y)[2]; for (i in 1:sl) s<-paste(s,"*",sep="")
 u<-NULL; sl<-35+10*dim(y)[2]; for (i in 1:sl) u<-paste(u,"-",sep="")
@@ -210,7 +237,7 @@ formatC(sum(d%*%(y^2)),digits=6,width=10,format="f"),"\n",sep="",file=ofile)
 cat(formatC(s,format="s"),"\n",file=ofile)
 }
 
-write.a<-function(a,ofile) {
+writeA<-function(a,ofile) {
 s<-NULL; sl<-35+10*dim(a)[2]; for (i in 1:sl) s<-paste(s,"*",sep="")
 u<-NULL; sl<-35+10*dim(a)[2]; for (i in 1:sl) u<-paste(u,"-",sep="")
 cat("Category Loadings\n",file=ofile)
@@ -226,7 +253,7 @@ formatC(sum(a^2),digits=6,width=10,format="f"),"\n",sep="",file=ofile)
 cat(formatC(s,format="s"),"\n",file=ofile)
 }
 
-weighted.gram.schmidt<-function(x,w) {
+weightedGramSchmidt<-function(x,w) {
 ss<-NULL; 
 for (j in 1:dim(x)[2]) {
 if (j > 1) {xx<-x[,1:(j-1)]; x[,j]<-x[,j]-xx%*%(crossprod(xx,(w*x[,j])))}
@@ -234,23 +261,23 @@ s<-sqrt(sum(w*x[,j]^2)); ss<-c(ss,s); x[,j]<-x[,j]/s;}
 list(pol=x,fac=ss)
 }
 
-orthogonal.polynomials<-function(w,x,p) {
-z<-weighted.gram.schmidt(outer(x,0:p,"^"),w)$pol[,2:(p+1)]
+orthogonalPolynomials<-function(w,x,p) {
+z<-weightedGramSchmidt(outer(x,0:p,"^"),w)$pol[,2:(p+1)]
 }
 
-center.x<-function(x,w) apply(x,2,function(z) z-weighted.mean(z,w))
+centerX<-function(x,w) apply(x,2,function(z) z-weighted.mean(z,w))
 
-norm.x<-function(x,w) {
+normX<-function(x,w) {
 qq<-La.svd(sqrt(w)*x); list(q=(1/sqrt(w))*(qq$u),r=qq$d)}
 
 pava<-function(x,w=rep(1,length(x)),block=weighted.mean){
 nblock<-n<-length(x); blocklist<-array(1:n,c(n,2)); blockvalues<-x; active<-1
 repeat{
-	if (!is.up.satisfied(blockvalues,active)) {
+	if (!isUpSatisfied(blockvalues,active)) {
 		blockmerge<-mergeBlockup(blocklist,blockvalues,x,w,active,block)
 		blockvalues<-blockmerge$v; blocklist<-blockmerge$l
 		nblock<-nblock-1
-		while (!is.down.satisfied(blockvalues,active)) {
+		while (!isDownSatisfied(blockvalues,active)) {
 			blockmerge<-mergeBlockup(blocklist,blockvalues,x,w,active-1,block)
 			blockvalues<-blockmerge$v; blocklist<-blockmerge$l; 
 			nblock<-nblock-1; active<-active-1;
@@ -258,7 +285,7 @@ repeat{
 		}
 	else if (active == nblock) break() else active<-active+1
 	}	
-put.back(n,blocklist,blockvalues)
+putBack(n,blocklist,blockvalues)
 }
 
 mergeBlockup<-function(blocklist,blockvalues,x,w,i,block){
@@ -272,25 +299,25 @@ blockvalues<-blockvalues[ii]
 list(v=blockvalues,l=blocklist)
 }
 
-put.back<-function(n,blocklist,blockvalues){
+putBack<-function(n,blocklist,blockvalues){
 x<-rep(0,n);nb<-length(blockvalues)
 for (i in 1:nb) {
 		x[blocklist[i,1]:blocklist[i,2]]<-blockvalues[i]}
 return(x)
 }
 
-is.up.satisfied<-function(x,i) (i == length(x))||(x[i]<=x[i+1])
+isUpSatisfied<-function(x,i) (i == length(x))||(x[i]<=x[i+1])
 
-is.down.satisfied<-function(x,i) (i == 1)||(x[i-1]<=x[i])
+isDownSatisfied<-function(x,i) (i == 1)||(x[i-1]<=x[i])
 
 tkhomals<-function(data)
 	{
 	require(tcltk,quietly=T); require(tkrplot,quietly=T)
-	ncat <- length(txtlabel <- c("active","rank","level","starplot","catplot","trfplot","lossplot","set"))
+	ncat <- length(txtlabel <- c("active","rank","level","starplot","catplot","trfplot","lossplot","hullplot","spanplot","set"))
 	tkvar<-allvars<-""
 	#need to have global window objects
 	output.Win <<- graphplot.Win <<- objplot.Win  <<- voronoi.Win <<- FALSE
-	starplots <- catplots <-trfplots <-lossplots <- FALSE
+	starplots <- catplots <-trfplots <-lossplots <- hullplots <- spanplots <- FALSE
 	active <- TRUE; sets <- 0; rank <- ndim <- 2; level <- "NO"
 	plotgraph <- tclVar(0)
 	plotobj <- tclVar(0)
@@ -314,8 +341,8 @@ tkhomals<-function(data)
 	tkpack(tframe, fill="x", side = "top")
 	tkpack(bfile, side="left")
 	tkconfigure(bfile, menu=bmenu)
-	tkadd (bmenu, "command", label="Quit", command=tkHom.quit)
-	tkpack(tklabel(top.frm, text="HomalsTk 1.0"))
+	tkadd (bmenu, "command", label="Quit", command=tkhomQuit)
+	tkpack(tklabel(top.frm, text="HomalsTk"))
 	graphplot.Label <- tklabel(right.frm, text="Graphplot")
 	graphplot.Button <- tkcheckbutton(right.frm, variable=plotgraph)
 	objplot.Label <- tklabel(right.frm, text="Objplot")
@@ -341,7 +368,9 @@ tkhomals<-function(data)
 	message <- paste("Data selected: ", name)
 	nvar<-dim(data)[2];
 	vname<-attr(data,"names") 
-	varlen <- max(nchar(vname))
+	varlen.vars <- max(nchar(vname))
+	varlen.char <- max(nchar("Variable"))
+	varlen <- max(varlen.vars, varlen.char)
 	
 	for(i in 1:nvar) {
 		label <- vname[i]
@@ -355,7 +384,9 @@ tkhomals<-function(data)
 		tclvalue(tkvar[5]) <- 0
 		tclvalue(tkvar[6]) <- 0
 		tclvalue(tkvar[7]) <- 0
-		tclvalue(tkvar[8]) <- i
+		tclvalue(tkvar[8]) <- 0
+		tclvalue(tkvar[9]) <- 0
+		tclvalue(tkvar[10]) <- i
 		}	
 
 	headerf <- tkframe(menu.Win, borderwidth=2)
@@ -371,11 +402,33 @@ tkhomals<-function(data)
 	varlen <- varlen * 12
 	
 	# Positioning checkboxes hack
-	x <- c(2,varlen,(varlen+50),(varlen+90),(varlen+130),(varlen+180),(varlen+230),(varlen+275),(varlen+330),(varlen+335))
+	x <- c(2,varlen,(varlen+50),(varlen+90),(varlen+130),(varlen+180),(varlen+230),(varlen+275),(varlen+330),(varlen+380),(varlen+440),(varlen+445))
 	y <- 2
+	varlen <- varlen + 1
+	x2 <- c(2,varlen,(varlen+50),(varlen+90),(varlen+130),(varlen+180),(varlen+230),(varlen+275),(varlen+330),(varlen+380),(varlen+440),(varlen+445))
+	y2 <- 45
 	variable.Label <- tklabel(top.can, text="Variable"); active.Label <- tklabel(top.can, text="Active"); rank.Label <- tklabel(top.can, text="Rank")
 	level.Label <- tklabel(top.can, text="Level"); starplot.Label <- tklabel(top.can, text="Starplot"); catplot.Label <- tklabel(top.can, text="Catplot")
-	trfplot.Label <- tklabel(top.can, text="Trfplot"); lossplot.Label <- tklabel(top.can, text="Lossplot"); sets.Label <- tklabel(top.can, text="Sets")
+	trfplot.Label <- tklabel(top.can, text="Trfplot"); lossplot.Label <- tklabel(top.can, text="Lossplot"); hullplot.Label <- tklabel(top.can, text="Hullplot")
+	spanplot.Label <- tklabel(top.can, text="Spanplot"); sets.Label <- tklabel(top.can, text="Sets")
+	
+	activecheck <- tclVar(1)
+	rankall <- tclVar(2)
+	levelall <- tclVar("NO")
+	starcheck <- tclVar(0)
+	catcheck <- tclVar(0)
+	trfcheck <- tclVar(0)
+	losscheck <- tclVar(0)
+	hullcheck <- tclVar(0)
+	spancheck <- tclVar(0)
+	
+	variable.Check <- tklabel(top.can, text="select all"); active.Check <- tkcheckbutton(top.can, variable=activecheck, command=function()tkhomCheckAll(activecheck,1))
+	rank.All <- tkentry(top.can, textvariable=rankall, width=3); level.All <- tkentry(top.can, textvariable=levelall, width=3)
+	starplot.Check <- tkcheckbutton(top.can, variable=starcheck, command=function()tkhomCheckAll(starcheck,4)); catplot.Check <- tkcheckbutton(top.can, variable=catcheck, command=function()tkhomCheckAll(catcheck,5))
+	trfplot.Check <- tkcheckbutton(top.can, variable=trfcheck, command=function()tkhomCheckAll(trfcheck,6)); lossplot.Check <- tkcheckbutton(top.can, variable=losscheck, command=function()tkhomCheckAll(losscheck,7))
+	hullplot.Check <- tkcheckbutton(top.can, variable=hullcheck, command=function()tkhomCheckAll(hullcheck,8)); spanplot.Check <- tkcheckbutton(top.can, variable=spancheck, command=function()tkhomCheckAll(spancheck,9))
+	
+	
 	tkcreate(top.can, "window", x[1], y, anchor = "nw", window = variable.Label)
 	tkcreate(top.can, "window", x[2], y, anchor = "nw", window = active.Label)
 	tkcreate(top.can, "window", x[3], y, anchor = "nw", window = rank.Label)
@@ -384,7 +437,23 @@ tkhomals<-function(data)
 	tkcreate(top.can, "window", x[6], y, anchor = "nw", window = catplot.Label)
 	tkcreate(top.can, "window", x[7], y, anchor = "nw", window = trfplot.Label)
 	tkcreate(top.can, "window", x[8], y, anchor = "nw", window = lossplot.Label)
-	tkcreate(top.can, "window", x[9], y, anchor = "nw", window = sets.Label)
+	tkcreate(top.can, "window", x[9], y, anchor = "nw", window = hullplot.Label)
+	tkcreate(top.can, "window", x[10], y, anchor = "nw", window = spanplot.Label)
+	tkcreate(top.can, "window", x[11], y, anchor = "nw", window = sets.Label)
+
+
+#Checkboxes to switch on/off related checkboxes
+	tkcreate(top.can, "window", x2[1], y2, anchor = "sw", window = variable.Check)
+	tkcreate(top.can, "window", x2[2], y2, anchor = "sw", window = active.Check)
+	#tkcreate(top.can, "window", x2[3], y2, anchor = "sw", window = rank.All)
+	#tkcreate(top.can, "window", x2[4], y2, anchor = "sw", window = level.All)
+	tkcreate(top.can, "window", x2[5], y2, anchor = "sw", window = starplot.Check)
+	tkcreate(top.can, "window", x2[6], y2, anchor = "sw", window = catplot.Check)
+	tkcreate(top.can, "window", x2[7], y2, anchor = "sw", window = trfplot.Check)
+	tkcreate(top.can, "window", x2[8], y2, anchor = "sw", window = lossplot.Check)
+	tkcreate(top.can, "window", x2[9], y2, anchor = "sw", window = hullplot.Check)
+	tkcreate(top.can, "window", x2[10], y2, anchor = "sw", window = spanplot.Check)
+
 	
 	for(i in 1:nvar) {
 		label <- vname[i]
@@ -393,7 +462,8 @@ tkhomals<-function(data)
 			}
 		variable.Header <- tklabel(can, text=label); active.Button <- tkcheckbutton(can, variable=tkvar[1]); rank.Entry <- tkentry(can, textvariable=tkvar[2], width=3)
 		level.Entry <- tkentry(can, textvariable=tkvar[3], width=3); starplot.Button <- tkcheckbutton(can, variable=tkvar[4]); catplot.Button <- tkcheckbutton(can, variable=tkvar[5])
-		trfplot.Button <- tkcheckbutton(can, variable=tkvar[6]); lossplot.Button <- tkcheckbutton(can, variable=tkvar[7]); sets.Entry <- tkentry(can, textvariable=tkvar[8], width=3)	
+		trfplot.Button <- tkcheckbutton(can, variable=tkvar[6]); lossplot.Button <- tkcheckbutton(can, variable=tkvar[7]); hullplot.Button <- tkcheckbutton(can, variable=tkvar[8])
+		spanplot.Button <- tkcheckbutton(can, variable=tkvar[9]); sets.Entry <- tkentry(can, textvariable=tkvar[10], width=3)	
 		tkcreate(can, "window", x[1], y, anchor = "nw", window = variable.Header)
 		tkcreate(can, "window", x[2], y, anchor = "nw", window = active.Button)
 		tkcreate(can, "window", x[3], y, anchor = "nw", window = rank.Entry)
@@ -402,32 +472,42 @@ tkhomals<-function(data)
 		tkcreate(can, "window", x[6], y, anchor = "nw", window = catplot.Button)
 		tkcreate(can, "window", x[7], y, anchor = "nw", window = trfplot.Button)
 		tkcreate(can, "window", x[8], y, anchor = "nw", window = lossplot.Button)
-		tkcreate(can, "window", x[9], y, anchor = "nw", window = sets.Entry)
+		tkcreate(can, "window", x[9], y, anchor = "nw", window = hullplot.Button)
+		tkcreate(can, "window", x[10], y, anchor = "nw", window = spanplot.Button)
+		tkcreate(can, "window", x[11], y, anchor = "nw", window = sets.Entry)
 		y <- y + 20
 		}
 	
-	rwidth = x[9] + 30
+	rwidth = x[11] + 30
 	top.rwidth = rwidth + 22
 	coord <- sprintf("0 0 %s %s", as.character(y), as.character(y))
 
 	fileinfo <- tklabel(main.frm, text=message, width=25, height=1)   
-	bsubmit <- tkbutton(bottom.frm, text="SUBMIT", command=function()tkHom.submit(data,name,plotgraph,plotobj,objlabel,objscores,voronoi,timer,saveme,ndim,nvar,ncat,allvars,tkvar,active,rank,level,starplots,catplots,trfplots,lossplots,sets))
+	bsubmit <- tkbutton(bottom.frm, text="SUBMIT", command=function()tkhomSubmit(data,name,plotgraph,plotobj,objlabel,objscores,voronoi,timer,saveme,ndim,nvar,ncat,allvars,tkvar,active,rank,level,starplots,catplots,trfplots,lossplots,hullplots,spanplots,sets))
 	tkpack(right.frm, fill="x", side = "left")
 	tkpack(top.frm, fill="x", side = "top")
 	tkpack(bottom.frm, fill="x", side="bottom")
 	tkpack(main.frm, fileinfo)
 
-	tkconfigure(top.can, width=top.rwidth, height=20)
+	tkconfigure(top.can, width=top.rwidth, height=40)
 	tkconfigure(can, width=rwidth, height=250,"-scrollregion",coord)
 	tkpack(tframe, fill="x", side = "top")
 	tkpack(headerf)
 	tkpack(froptions)
 	tkpack(bsubmit)
 	tkfocus(menu.Win)
+	
+	tkhomCheckAll <- function(var,index) { 
+		k <- as.numeric(tclvalue(var))
+		for(i in 1:nvar) {
+			j <- index + (ncat * (i -1))
+			if (k==0) { tclvalue(allvars[j]) <- 0 } else { tclvalue(allvars[j]) <- 1 }
+			}
+		}
 	}
 
-tkHom.submit <- function(data,name,plotgraph,plotobj,objlabel,objscores,voronoi,timer,saveme,ndim,nvar,ncat,allvars,tkvar,active,rank,level,starplots,catplots,trfplots,lossplots,sets) {
-    if (data!="") { 
+
+tkhomSubmit <- function(data,name,plotgraph,plotobj,objlabel,objscores,voronoi,timer,saveme,ndim,nvar,ncat,allvars,tkvar,active,rank,level,starplots,catplots,trfplots,lossplots,hullplots,spanplots,sets) {
     	graphplot.Image <- objplot.Image <- voronoi.Image <- ""
     	graphplot <- as.numeric(tclvalue(plotgraph)); objplot <- as.numeric(tclvalue(plotobj)); objlabel <- as.numeric(tclvalue(objlabel))
 		objscores <- as.numeric(tclvalue(objscores)); voronoi <- as.numeric(tclvalue(voronoi)); timer <- as.numeric(tclvalue(timer)); saveme <- as.numeric(tclvalue(saveme))
@@ -445,7 +525,11 @@ tkHom.submit <- function(data,name,plotgraph,plotobj,objlabel,objscores,voronoi,
 			if (arr4==1) { trfplots[i] <- TRUE } else { trfplots[i] <- FALSE }
 			arr5 <- as.numeric(tclvalue(allvars[7 + (ncat * (i-1))]))
 			if (arr5==1) { lossplots[i] <- TRUE } else { lossplots[i] <- FALSE }
-            sets[i] <- as.numeric(tclvalue(allvars[8 + (ncat * (i-1))]))
+			arr6 <- as.numeric(tclvalue(allvars[8 + (ncat * (i-1))]))
+			if (arr6==1) { hullplots[i] <- TRUE } else { hullplots[i] <- FALSE }
+			arr7 <- as.numeric(tclvalue(allvars[9 + (ncat * (i-1))]))
+			if (arr7==1) { spanplots[i] <- TRUE } else { spanplots[i] <- FALSE }
+            sets[i] <- as.numeric(tclvalue(allvars[10 + (ncat * (i-1))]))
 		 	}
 		if (graphplot) {
 			if (is.tkwin(graphplot.Win)) tkdestroy(graphplot.Win)
@@ -468,7 +552,7 @@ tkHom.submit <- function(data,name,plotgraph,plotobj,objlabel,objscores,voronoi,
 			voronoi.Image <- tkrplot(voronoi.Win, function() plot(0,0))
 			tkgrid(voronoi.Image)
             }
-        homals(data, sets = sets, ndim = ndim, active = active, rank = rank, level = level, starplots = starplots, catplots = catplots, trfplots = trfplots, lossplots = lossplots, graphplot = graphplot, objplot = objplot, objscores = objscores, objlabel = objlabel, voronoi = voronoi, timer = timer, save.me = saveme, tk = TRUE, img1 = graphplot.Image, img2 = objplot.Image, img3 = voronoi.Image, name = name)
+        homals(data, sets = sets, ndim = ndim, active = active, rank = rank, level = level, starplots = starplots, catplots = catplots, trfplots = trfplots, lossplots = lossplots, hullplots = hullplots, spanplots = spanplots, graphplot = graphplot, objplot = objplot, objscores = objscores, objlabel = objlabel, voronoi = voronoi, timer = timer, saveMe = saveme, tk = TRUE, img1 = graphplot.Image, img2 = objplot.Image, img3 = voronoi.Image, name = name)
 		if (is.tkwin(output.Win)) tkdestroy(output.Win)
 		output.Win <<- tktoplevel()
 		tktitle(output.Win) <- "Results"
@@ -483,10 +567,9 @@ tkHom.submit <- function(data,name,plotgraph,plotobj,objlabel,objscores,voronoi,
 		out <- scan(filename, what="", sep="\n")
 		tkdelete(view, 0, "end")
     	tkinsert(view, "end", out)
-		}
     }
     
-tkHom.quit <- function() {
+tkhomQuit <- function() {
 	tkdestroy(menu.Win)
 	if (is.tkwin(output.Win)) tkdestroy(output.Win) 
 	if (is.tkwin(graphplot.Win)) tkdestroy(graphplot.Win)
@@ -504,6 +587,8 @@ homals<-function(data,                 # data (in data-frame)
                   catplots=F,          # which category plots (default none)
                   trfplots=F,          # which transformation plots (default none)
                   lossplots=F,         # which loss plots (default none)
+                  hullplots=F,		   # which hullplots (default none)
+                  spanplots=F,		   # which spanning tree plots (default none)
                   graphplot=F,         # graphplot (default no)
                   objplot=F,           # object score plot (default no)
                   objscores=F,         # object scores written to file (default no)
@@ -513,7 +598,7 @@ homals<-function(data,                 # data (in data-frame)
                   eps2=1e-6,           # iteration precision eigenvectors (default 1e-6)
                   itermax=100,         # maximum number of iterations (default 100)
                   voronoi=F,           # voronoi diagram
-                  save.me=F,           # do we return the results
+                  saveMe=F,            # do we return the results
                   demo=F,              # animated iteration demo (default no)
                   timer=F,             # time the steps of program (default no)
                   tk=F,				   # create tk homals output (default no)
@@ -542,7 +627,9 @@ if (length(starplots)==1) starplots<-rep(starplots,nvar)
 if (length(catplots)==1) catplots<-rep(catplots,nvar)
 if (length(trfplots)==1) trfplots<-rep(trfplots,nvar)
 if (length(lossplots)==1) lossplots<-rep(lossplots,nvar)
-do.pdf<-any(starplots,catplots,trfplots,lossplots,graphplot,objplot,voronoi)
+if (length(hullplots)==1) hullplots<-rep(hullplots,nvar)
+if (length(spanplots)==1) spanplots<-rep(spanplots,nvar)
+doPDF<-any(starplots,catplots,trfplots,lossplots,hullplots,spanplots,graphplot,objplot,voronoi)
 if (length(rank)==1) rank<-rep(rank,nvar)
 if (length(level)==1) level<-rep(level,nvar)
 for (j in 1:nvar) {
@@ -552,19 +639,19 @@ for (j in 1:nvar) {
 outfile<-file(paste(name,"out",sep="."),"w")
 if (objscores)
 	objfile<-file(paste(name,"obj",sep="."),"w")
-x<-cbind(orthogonal.polynomials(mis,1:nobj,ndim))
-x<-norm.x(center.x(x,mis),mis)$q
-y<-lapply(1:nvar, function(j) compute.y(data[,j],x))
+x<-cbind(orthogonalPolynomials(mis,1:nobj,ndim))
+x<-normX(centerX(x,mis),mis)$q
+y<-lapply(1:nvar, function(j) computeY(data[,j],x))
 if (demo) plot(x, col="GREEN", pch=8)
 if (timer) itime<-proc.time()
 cat("Iterations:\n",file=outfile)
 repeat {
 	iter<-iter+1
 	totsum<-array(0.0,dim(x))
-	if (pca) up.y<-pca.update.y(data,x,y,totsum,active,rank,level,nset)
-		else up.y<-cca.update.y(data,x,y,totsum,active,rank,level,sets)
+	if (pca) up.y<-pcaUpdateY(data,x,y,totsum,active,rank,level,nset)
+		else up.y<-ccaUpdateY(data,x,y,totsum,active,rank,level,sets)
 	y<-up.y$y; totsum<-up.y$totsum
-	qv<-norm.x(center.x((1/mis)*totsum,mis),mis)
+	qv<-normX(centerX((1/mis)*totsum,mis),mis)
 	z<-qv$q;r<-qv$r;ops=sum(r);aps<-sum(La.svd(crossprod(x,mis*z),0,0)$d)/ndim	
 	cat("Iteration: ",formatC(iter,digits=3,width=3)," Eigenvalues: ",
 		formatC(r,digits=6,width=9,format="f"),
@@ -579,7 +666,7 @@ repeat {
 	if (((ops - pops) < eps1) || ((1.0 - aps) < eps2) || (iter == itermax)) break 
 		else {x<-z; pops<-ops}}
 if (timer) ctime<-proc.time()
-if (do.pdf)
+if (doPDF)
 	pdf(file=paste(name,"pdf",sep="."),encoding="MacRoman")
 xlim<-c(min(z[,1]),max(z[,1])); if (ndim > 1) ylim<-c(min(z[,2]),max(z[,2]))
 if (graphplot) if (ndim > 1) for (s in 1:(ndim-1)) for (t in (s+1):ndim) {
@@ -599,26 +686,34 @@ if (voronoi) {
 	}
 ylist<-alist<-clist<-ulist<-NULL
 for (j in 1:nvar) {
-    gg<-data[,j]; c<-compute.y(gg,z); d<-as.vector(table(gg))
-    lst<-restrict.y(d,c,rank[j],level[j])
+    gg<-data[,j]; c<-computeY(gg,z); d<-as.vector(table(gg))
+    lst<-restrictY(d,c,rank[j],level[j])
     y<-lst$y; a<-lst$a; u<-lst$z
     ylist<-c(ylist,list(y)); alist<-c(alist,list(a)); clist<-c(clist,list(c)); ulist<-c(ulist,list(u))
-    write.head(name,vname[j],ndim,active[j],rank[j],level[j],outfile)
-    write.y(data[,j],y,"Y",outfile)
-    write.y(data[,j],c,"C",outfile)
-    write.y(data[,j],u,"Z",outfile)
-    write.a(t(a),outfile)
+    writeHead(name,vname[j],ndim,active[j],rank[j],level[j],outfile)
+    writeY(data[,j],y,"Y",outfile)
+    writeY(data[,j],c,"C",outfile)
+    writeY(data[,j],u,"Z",outfile)
+    writeA(t(a),outfile)
 	if (starplots[j]) {
 		if (ndim > 1) for (s in 1:(ndim-1)) for (t in (s+1):ndim) {
 			starplot(name,vname[j],data[,j],y[,c(s,t)],z[,c(s,t)],s,t)}}
-	if (lossplots[j]) {
-		if (ndim > 1) for (s in 1:(ndim-1)) for (t in (s+1):ndim) {
-			lossplot(name,vname[j],data[,j],y[,c(s,t)],z[,c(s,t)],s,t)}}
 	if (catplots[j]) {
 		if (ndim > 1) for (s in 1:(ndim-1)) for (t in (s+1):ndim) {
 			catplot(name,vname[j],data[,j],y[,c(s,t)],xlim,ylim,s,t)}}
-	if (trfplots[j]) trfplot(name,vname[j],data[,j],y)}
-if (do.pdf) dev.off()
+	if (trfplots[j]) trfplot(name,vname[j],data[,j],y)
+	if (lossplots[j]) {
+		if (ndim > 1) for (s in 1:(ndim-1)) for (t in (s+1):ndim) {
+			lossplot(name,vname[j],data[,j],y[,c(s,t)],z[,c(s,t)],s,t)}}
+	if (hullplots[j]) {
+		if (ndim > 1) for (s in 1:(ndim-1)) for (t in (s+1):ndim) {
+			hullplot(name,vname[j],data[,j],y[,c(s,t)],z[,c(s,t)],s,t)}}
+	if (spanplots[j]) {
+		require("ape",quietly=T)
+		if (ndim > 1) for (s in 1:(ndim-1)) for (t in (s+1):ndim) {
+			spanplot(name,vname[j],data[,j],y[,c(s,t)],z[,c(s,t)],s,t)}}
+	}
+if (doPDF) dev.off()
 if (objscores){
 	write(x,file=objfile,ncolumns=p)
 	close(objfile)}
@@ -628,6 +723,6 @@ if (timer) {otime<-proc.time()
 	cat("Iteration time: ",formatC((ctime-itime)/iter,digits=6,width=10,format="f"),"\n",file=outfile)
 	cat("Output time:    ",formatC(otime-ctime,digits=6,width=10,format="f"),"\n",file=outfile)}
 close(outfile)
-if (save.me) return(x=z,y=ylist,c=clist,a=alist,z=ulist)
+if (saveMe) return(x=z,y=ylist,c=clist,a=alist,z=ulist)
 }
 
